package com.github.binarywang.demo.wx.cp.controller;

import com.github.binarywang.demo.wx.cp.config.WxCpConfiguration;
import com.github.binarywang.demo.wx.cp.model.ExternalUser;
import com.github.binarywang.demo.wx.cp.service.impl.ExternalUserService;
import com.github.binarywang.demo.wx.cp.utils.JsonUtils;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.cp.api.WxCpService;
import me.chanjar.weixin.cp.bean.WxCpUser;
import me.chanjar.weixin.cp.bean.WxCpUserExternalContactInfo;
import me.chanjar.weixin.cp.bean.WxCpXmlMessage;
import me.chanjar.weixin.cp.bean.WxCpXmlOutMessage;
import me.chanjar.weixin.cp.util.crypto.WxCpCryptUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/wx/cp/addcontact")
public class WxaAddContactController {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private static Integer agentId=1000003;
    final WxCpService wxCpService = WxCpConfiguration.getCpService(agentId);
    @Autowired
    private ExternalUserService externalUserService;
    /**
     * 企业微信接收客户变更事件验证接口回调路径
     * @return
     */
    @GetMapping("/authWx")
    public String authGet(@RequestParam(name = "msg_signature", required = false) String signature,
                          @RequestParam(name = "timestamp", required = false) String timestamp,
                          @RequestParam(name = "nonce", required = false) String nonce,
                          @RequestParam(name = "echostr", required = false) String echostr) {
        this.logger.info("\n1接收到来自微信服务器的认证消息：signature = [{}], timestamp = [{}], nonce = [{}], echostr = [{}]"+signature+timestamp+nonce+echostr);

        if (StringUtils.isAnyBlank(signature, timestamp, nonce, echostr)) {
            throw new IllegalArgumentException("请求参数非法，请核实!");
        }
        if (wxCpService == null) {
            throw new IllegalArgumentException(String.format("未找到对应agentId=[%d]的配置，请核实！", agentId));
        }

        if (wxCpService.checkSignature(signature, timestamp, nonce, echostr)) {
            return new WxCpCryptUtil(wxCpService.getWxCpConfigStorage()).decrypt(echostr);
        }

        return "非法请求";
    }

    @PostMapping(value = "/authWx")
    public String authPost(@RequestBody String requestBody,
                           @RequestParam(name = "msg_signature", required = false) String signature,
                           @RequestParam(name = "timestamp", required = false) String timestamp,
                           @RequestParam(name = "nonce", required = false) String nonce,
                           @RequestParam(name = "echostr", required = false) String echostr) {
        this.logger.info("\n1接收到来自微信服务器的认证消息："+signature+"--"+timestamp+"--"+nonce+"--"+echostr);


        //获取Token
        try {
            Date date = new Date();
            WxCpXmlMessage inMessage = WxCpXmlMessage.fromEncryptedXml(requestBody, wxCpService.getWxCpConfigStorage(),
                timestamp, nonce, signature);
            this.logger.info("\n消息解密后内容为：\n{} "+JsonUtils.toJson(inMessage));
            WxCpXmlOutMessage outMessage = this.route(agentId, inMessage);
            if (outMessage == null) {
                return "";
            }

            //获取外部客户信息
            WxCpUserExternalContactInfo exuser = wxCpService.getExternalContactService().getContactDetail(inMessage.getExternalUserId());

            //企业人信息
            WxCpUser wxUser = wxCpService.getUserService().getById(inMessage.getUserId());
            //添加好友回调add_external_contact
            if(inMessage.getChangeType().equals("add_external_contact")){
                List<WxCpUserExternalContactInfo.FollowedUser> follList=exuser.getFollowedUsers();
                for (WxCpUserExternalContactInfo.FollowedUser foll:follList) {
                    if(foll.getRemarkMobiles()!=null && foll.getRemarkMobiles().length>0){
                        //是否存在
                        int count =externalUserService.getExternalCount(foll.getRemarkMobiles()[0]);
                        if(count<=0){
                            //保存用户信息
                            ExternalUser externalUser=new ExternalUser();
                            externalUser.setAvatar(exuser.getExternalContact().getAvatar());
                            externalUser.setExternalUserid(exuser.getExternalContact().getExternalUserId());
                            externalUser.setName(exuser.getExternalContact().getName());
                            externalUser.setGender(String.valueOf(exuser.getExternalContact().getGender()));
                            externalUser.setType(exuser.getExternalContact().getType());
                            externalUser.setUserid(foll.getUserId());
                            externalUser.setCreatetime(date);
                            externalUser.setRemarkMobiles(foll.getRemarkMobiles()[0]);

                            externalUser.setCompanyAvatar(wxUser.getAvatar());
                            externalUser.setCompanyGender(wxUser.getGender().getGenderName());
                            externalUser.setCompanyMobiles(wxUser.getMobile());
                            externalUser.setCompanyName(wxUser.getName());
                            externalUser.setTermainType("企业微信");
                            //企业微信未返回年龄，默认0
                            externalUser.setUserAge(0);
                            externalUserService.updateExternalUser(externalUser);
                        }else{
                            this.logger.info("已存在用户：",foll.getRemarkMobiles()[0]);
                        }
                        this.logger.info("通过手机号查找的外部客户：",foll.getRemarkMobiles()[0]);
                    }else{
                        this.logger.info("不是通过手机号查找的外部客户：",exuser.getExternalContact().getName());
                    }
                }
            }else{//删除好友回调del_follow_user

            }
        } catch (WxErrorException e) {
            e.printStackTrace();
        }
        return requestBody;
    }

    private WxCpXmlOutMessage route(Integer agentId, WxCpXmlMessage message) {
        try {
            return WxCpConfiguration.getRouters().get(agentId).route(message);
        } catch (Exception e) {
            this.logger.error(e.getMessage(), e);
        }

        return null;
    }
}
